import {
  select,
  range,
  symbols,
  symbol,
} from 'd3';

const width = window.innerWidth;
const height = window.innerHeight;

// create a new SVG element and append it to the body of the document
// set width and height
const svg = select('body')
  .append('svg')
  .attr('width', width)
  .attr('height', height);

// horizontal rectangles in background
const n = 100;
svg
  .selectAll('rect.horizontal') // select all rectangles on the page of class horizontal
  .data(range(n)) // create one rectangle for each mark
  .join('rect')
  .attr('y', (d) => d * 20)
  .attr('width', width)
  .attr('height', 10)
  .attr('class', 'horizontal')
  .attr('mask', 'url(#mask-h-b)');

// vertical rectangles in foreground
svg
  .selectAll('rect.vertical') // select all rectangles on the page of class vertical
  .data(range(n)) // create one rectangle for each mark
  .join('rect')
  .attr('x', (d) => d * 20)
  .attr('width', 10)
  .attr('height', height)
  .attr('class', 'vertical')
  .attr('mask', 'url(#mask-v-f)');

// horizontal rectangles in foreground
svg
  .selectAll('rect.horizontal') // select all rectangles on the page of class horizontal
  .data(range(n)) // create one rectangle for each mark
  .join('rect')
  .attr('y', (d) => d * 20)
  .attr('width', width)
  .attr('height', 10)
  .attr('class', 'horizontal')
  .attr('mask', 'url(#mask-b-f)');

// vertical rectangles in background
svg
  .selectAll('rect.vertical') // select all rectangles on the page of class vertical
  .data(range(n)) // create one rectangle for each mark
  .join('rect')
  .attr('x', (d) => d * 20)
  .attr('width', 10)
  .attr('height', height)
  .attr('class', 'vertical')
  .attr('mask', 'url(#mask-v-b)');

const renderMask = (selection, id, inverted) => {
  // mask element
  const mask = selection
    .append('mask')
    .attr('id', id);

  // rectangle mask - odd shapes
const r1 = mask
    .append('g') // shape is initially centered at 0,0; use group element to move the shape around
    .attr(
      'transform',
      `translate(${width / 2}, ${height / 2})`
    );
  mask
    .selectAll('r1')
    .data([1, 3, 5])
    .join((enter) =>
      enter
        .append('rect')
        .attr('width', width / 6)
        .attr('height', height)
        .attr(
          'transform',
          (d) =>
            `translate(${d * 125 + 25}, ${0})`
        )
        .attr(
          'fill',
          inverted ? 'white' : 'black'
        )
    );


  // rectangle mask - even shapes
  const r2 = mask
    .append('g') // shape is initially centered at 0,0; use group element to move the shape around
    .attr(
      'transform',
      `translate(${width / 2}, ${height / 2})`
    );
  mask
    .selectAll('r2')
    .data([0, 2, 4, 6])
    .join((enter) =>
      enter
        .append('rect')
        .attr('width', width / 6)
        .attr('height', height)
        .attr(
          'transform',
          (d) =>
            `translate(${d * 125 + 25}, ${0})`
        )
        .attr(
          'fill',
          inverted ? 'black' : 'white'
        )
    );

  // odd shapes
  const s1 = mask
    .append('g') // shape is initially centered at 0,0; use group element to move the shape around
    .attr(
      'transform',
      `translate(${width / 2}, ${height / 2})`
    );
  mask
    .selectAll('s1')
    .data([1, 3, 5])
    .join((enter) =>
      enter
        .append('g')
        .append('path')
        .attr('d', (d) =>
          symbol(symbols[d], 10000)()
        )
        .attr(
          'transform',
          (d) =>
            `translate(${d * 125 + 100}, ${
              height / 2
            })`
        )
        .attr(
          'fill',
          inverted ? 'black' : 'white'
        )
    );

  // even shapes
  const s2 = mask
    .append('g') // shape is initially centered at 0,0; use group element to move the shape around
    .attr(
      'transform',
      `translate(${width / 2}, ${height / 2})`
    );
  mask
    .selectAll('s2')
    .data([0, 2, 4, 6])
    .join((enter) =>
      enter
        .append('g')
        .append('path')
        .attr('d', (d) =>
          symbol(symbols[d], 10000)()
        )
        .attr(
          'transform',
          (d) =>
            `translate(${d * 125 + 100}, ${
              height / 2
            })`
        )
        .attr(
          'fill',
          inverted ? 'white' : 'black'
        )
    );
};

svg
  .call(renderMask, 'mask-h-b', true) // rect
  .call(renderMask, 'mask-v-f', false) // shape
  .call(renderMask, 'mask-v-b', false) // rect
  .call(renderMask, 'mask-h-f', true); // shape


// goal:
// odd shapes have horizontal lines in the back and vertical lines in the front

// even shapes have vertical lines in the back and horizontal lines in the front
